==> 1.0.3.tar.gz: lib/src/ws/ws_client.dart <==
import 'dart:async';

import 'package:dio/dio.dart';
import 'package:enum_to_string/enum_to_string.dart';
import 'package:pool/pool.dart';
import 'package:shadertoy_api/shadertoy_api.dart';
import 'package:shadertoy_client/src/http_client.dart';
import 'package:shadertoy_client/src/ws/ws_options.dart';

/// The base Shadertoy Client API for WS access
abstract class ShadertoyWS extends ShadertoyClient {}

/// A Shadertoy REST API client
///
/// Provides an implementation of the [ShadertoyWS] thus allowing the creation
/// of a client to access all the methods provided by the shadertoy REST API
/// as described in the Shadertoy [howto](https://www.shadertoy.com/howto)
class ShadertoyWSClient extends ShadertoyHttpClient<ShadertoyWSOptions>
    implements ShadertoyWS {
  /// Creates a [ShadertoyWSClient]
  ///
  /// * [options]: The [ShadertoyWSOptions] used to configure this client
  /// * [client]: A pre-initialized [Dio] client
  ShadertoyWSClient(ShadertoyWSOptions options, {Dio client})
      : super(options, client: client);

  /// Finds a [Shader] by Id
  ///
  /// [shaderId]: The id of the shader
  ///
  /// Returns a [FindShaderResponse] with the [Shader] or a [ResponseError]
  Future<FindShaderResponse> _getShaderById(String shaderId) {
    return client.get('${options.apiPath}/shaders/$shaderId', queryParameters: {
      'key': options.apiKey
    }).then((Response<dynamic> response) => jsonResponse<FindShaderResponse>(
        response, (data) => FindShaderResponse.fromJson(data),
        context: CONTEXT_SHADER, target: shaderId));
  }

  @override
  Future<FindShaderResponse> findShaderById(String shaderId) {
    return catchDioError<FindShaderResponse>(
        _getShaderById(shaderId),
        (de) => FindShaderResponse(
            error: toResponseError(de,
                context: CONTEXT_SHADER, target: shaderId)));
  }

  /// Finds shaders by ids
  ///
  /// * [shaderIds]: The list of shaders
  ///
  /// Returns a [FindShadersResponse] with the list of [Shader] obtained or a [ResponseError]
  Future<FindShadersResponse> _getShadersByIdSet(Set<String> shaderIds) {
    var shaderTaskPool = Pool(options.poolMaxAllocatedResources,
        timeout: Duration(seconds: options.poolTimeout));

    return Future.wait(shaderIds
            .map((id) => pooledRetry(shaderTaskPool, () => findShaderById(id))))
        .then((l) => FindShadersResponse(shaders: l));
  }

  @override
  Future<FindShadersResponse> findShadersByIdSet(Set<String> shaderIds) {
    return _getShadersByIdSet(shaderIds);
  }

  /// Finds shader ids
  ///
  /// * [term]: Shaders that have [term] in the name or in description
  /// * [filters]: A set of tag filters
  /// * [sort]: The sort order of the shaders
  /// * [from]: A 0 based index for results returned
  /// * [num]: The total number of results
  ///
  /// Returns a [FindShaderIdsResponse] with a list of ids or a [ResponseError]
  Future<FindShaderIdsResponse> _getShaderIds(
      {String term, Set<String> filters, Sort sort, int from, int num}) {
    var sb = StringBuffer('${options.apiPath}/shaders/query');

    if (term != null && term.isNotEmpty) {
      sb.write('/$term');
    }

    sb.write('?key=${options.apiKey}');

    if (filters != null) {
      for (var filter in filters) {
        sb.write('&filter=$filter');
      }
    }

    if (sort != null) {
      sb.write('&sort=${EnumToString.convertToString(sort)}');
    }

    if (from != null) {
      sb.write('&from=$from');
    }

    if (num != null) {
      sb.write('&num=$num');
    }

    return client.get(sb.toString()).then((Response<dynamic> response) =>
        jsonResponse<FindShaderIdsResponse>(
            response, (data) => FindShaderIdsResponse.fromJson(data)));
  }

  @override
  Future<FindShadersResponse> findShaders(
      {String term, Set<String> filters, Sort sort, int from, int num}) {
    return catchDioError<FindShadersResponse>(
        _getShaderIds(
                term: term,
                filters: filters,
                sort: sort,
                from: from,
                num: num ?? options.shaderCount)
            .then((r) => _getShadersByIdSet(r.ids.toSet())),
        (de) => FindShadersResponse(
            error: toResponseError(de, context: CONTEXT_SHADER)));
  }

  @override
  Future<FindShaderIdsResponse> findAllShaderIds() {
    return catchDioError<FindShaderIdsResponse>(
        client.get('${options.apiPath}/shaders', queryParameters: {
          'key': options.apiKey
        }).then((Response<dynamic> response) =>
            jsonResponse<FindShaderIdsResponse>(
                response, (data) => FindShaderIdsResponse.fromJson(data))),
        (de) => FindShaderIdsResponse(
            error: toResponseError(de, context: CONTEXT_SHADER)));
  }

  @override
  Future<FindShaderIdsResponse> findShaderIds(
      {String term, Set<String> filters, Sort sort, int from, int num}) {
    return catchDioError<FindShaderIdsResponse>(
        _getShaderIds(
            term: term,
            filters: filters,
            sort: sort,
            from: from,
            num: num ?? options.shaderCount),
        (de) => FindShaderIdsResponse(
            error: toResponseError(de, context: CONTEXT_SHADER)));
  }
}

/// Creates [ShadertoyWS] backed by the Shadertoy Rest API
///
/// * [apiKey]: The API key
/// * [apiPath]: The base api path
/// * [baseUrl]: The Shadertoy base url
/// * [poolMaxAllocatedResources]: The maximum number of resources allocated for parallel calls
/// * [poolTimeout]: The timeout before giving up on a call
/// * [retryMaxAttempts]: The maximum number of attempts at a failed request
/// * [shaderCount]: The number of shaders fetched in a paged call
/// * [errorHandling]: The error handling mode
/// * [client]: A pre-initialized [Dio] client
ShadertoyWS newShadertoyWSClient(String apiKey,
    {String apiPath,
    String baseUrl,
    int poolMaxAllocatedResources,
    int poolTimeout,
    int retryMaxAttempts,
    int shaderCount,
    ErrorMode errorHandling,
    Dio client}) {
  return ShadertoyWSClient(
      ShadertoyWSOptions(
          apiKey: apiKey,
          apiPath: apiPath,
          baseUrl: baseUrl,
          poolMaxAllocatedResources: poolMaxAllocatedResources,
          poolTimeout: poolTimeout,
          retryMaxAttempts: retryMaxAttempts,
          shaderCount: shaderCount,
          errorHandling: errorHandling),
      client: client);
}

==> 2.0.1.tar.gz: lib/src/ws/ws_client.dart <==
import 'dart:async';

import 'package:dio/dio.dart';
import 'package:enum_to_string/enum_to_string.dart';
import 'package:pool/pool.dart';
import 'package:shadertoy/shadertoy_api.dart';
import 'package:shadertoy_client/src/http_client.dart';
import 'package:shadertoy_client/src/ws/ws_options.dart';

/// The base Shadertoy Client API for WS access
abstract class ShadertoyWS extends ShadertoyClient {}

/// A Shadertoy REST API client
///
/// Provides an implementation of the [ShadertoyWS] thus allowing the creation
/// of a client to access all the methods provided by the shadertoy REST API
/// as described in the Shadertoy [howto](https://www.shadertoy.com/howto)
class ShadertoyWSClient extends ShadertoyHttpClient<ShadertoyWSOptions>
    implements ShadertoyWS {
  /// Creates a [ShadertoyWSClient]
  ///
  /// * [options]: The [ShadertoyWSOptions] used to configure this client
  /// * [client]: A pre-initialized [Dio] client
  ShadertoyWSClient(ShadertoyWSOptions options, {Dio? client})
      : super(options, client: client);

  /// Finds a [Shader] by Id
  ///
  /// [shaderId]: The id of the shader
  ///
  /// Returns a [FindShaderResponse] with the [Shader] or a [ResponseError]
  Future<FindShaderResponse> _getShaderById(String shaderId) {
    return client.get('${options.apiPath}/shaders/$shaderId', queryParameters: {
      'key': options.apiKey
    }).then((Response<dynamic> response) => jsonResponse<FindShaderResponse>(
        response, (data) => FindShaderResponse.fromJson(data),
        context: contextShader, target: shaderId));
  }

  @override
  Future<FindShaderResponse> findShaderById(String shaderId) {
    return catchDioError<FindShaderResponse>(
        _getShaderById(shaderId),
        (de) => FindShaderResponse(
            error:
                toResponseError(de, context: contextShader, target: shaderId)));
  }

  /// Finds shaders by ids
  ///
  /// * [shaderIds]: The list of shaders
  ///
  /// Returns a [FindShadersResponse] with the list of [Shader] obtained or a [ResponseError]
  Future<FindShadersResponse> _getShadersByIdSet(Set<String> shaderIds) {
    var shaderTaskPool = Pool(options.poolMaxAllocatedResources,
        timeout: Duration(seconds: options.poolTimeout));

    return Future.wait(shaderIds
            .map((id) => pooledRetry(shaderTaskPool, () => findShaderById(id))))
        .then((l) => FindShadersResponse(shaders: l));
  }

  @override
  Future<FindShadersResponse> findShadersByIdSet(Set<String> shaderIds) {
    return _getShadersByIdSet(shaderIds);
  }

  /// Finds shader ids
  ///
  /// * [term]: Shaders that have [term] in the name or in description
  /// * [filters]: A set of tag filters
  /// * [sort]: The sort order of the shaders
  /// * [from]: A 0 based index for results returned
  /// * [num]: The total number of results
  ///
  /// Returns a [FindShaderIdsResponse] with a list of ids or a [ResponseError]
  Future<FindShaderIdsResponse> _getShaderIds(
      {String? term, Set<String>? filters, Sort? sort, int? from, int? num}) {
    var sb = StringBuffer('${options.apiPath}/shaders/query');

    if (term != null && term.isNotEmpty) {
      sb.write('/$term');
    }

    sb.write('?key=${options.apiKey}');

    if (filters != null) {
      for (var filter in filters) {
        sb.write('&filter=$filter');
      }
    }

    if (sort != null) {
      sb.write('&sort=${EnumToString.convertToString(sort)}');
    }

    if (from != null) {
      sb.write('&from=$from');
    }

    if (num != null) {
      sb.write('&num=$num');
    }

    return client.get(sb.toString()).then((Response<dynamic> response) =>
        jsonResponse<FindShaderIdsResponse>(
            response, (data) => FindShaderIdsResponse.fromJson(data)));
  }

  @override
  Future<FindShadersResponse> findShaders(
      {String? term, Set<String>? filters, Sort? sort, int? from, int? num}) {
    return catchDioError<FindShadersResponse>(
        _getShaderIds(
                term: term,
                filters: filters,
                sort: sort,
                from: from,
                num: num ?? options.shaderCount)
            .then((r) => _getShadersByIdSet((r.ids ?? []).toSet())),
        (de) => FindShadersResponse(
            error: toResponseError(de, context: contextShader)));
  }

  @override
  Future<FindShaderIdsResponse> findAllShaderIds() {
    return catchDioError<FindShaderIdsResponse>(
        client.get('${options.apiPath}/shaders', queryParameters: {
          'key': options.apiKey
        }).then((Response<dynamic> response) =>
            jsonResponse<FindShaderIdsResponse>(
                response, (data) => FindShaderIdsResponse.fromJson(data))),
        (de) => FindShaderIdsResponse(
            error: toResponseError(de, context: contextShader)));
  }

  @override
  Future<FindShaderIdsResponse> findShaderIds(
      {String? term, Set<String>? filters, Sort? sort, int? from, int? num}) {
    return catchDioError<FindShaderIdsResponse>(
        _getShaderIds(
            term: term,
            filters: filters,
            sort: sort,
            from: from,
            num: num ?? options.shaderCount),
        (de) => FindShaderIdsResponse(
            error: toResponseError(de, context: contextShader)));
  }
}
